program main

  use vtype_module
  use netcdf
  use ni3_module

  use fi_module

  use io_profile, only : IOProfileOutput

  implicit none

  real(dp)                :: Ps
  real(dp)                :: Ts

  integer                 :: NLev
  real(dp), allocatable   :: r_Press(:)
  integer                 :: NMol
  integer , allocatable   :: m_MolNum(:)             ! Molecular number
  real(dp), allocatable   :: r_Temp  (:)
  real(dp), allocatable   :: rm_VMR  (:,:)


  character(128) :: ctlfn = 'mkprofile.conf'

  character(128) :: InASCIIFn
  character(128) :: OutNcFn


  integer             , parameter   :: NChar = stdstr
  real(8)             , allocatable :: z_Press      (:)
  integer                           :: NPtcl
  integer             , allocatable :: a_PtclNum    (:)
  character(len=NChar), allocatable :: a_PtclName   (:)
  real(8)             , allocatable :: a_PtclDens   (:)
  real(8)             , allocatable :: za_PtclEffRad(:,:)
  real(8)             , allocatable :: za_PtclMMR   (:,:)

  integer :: FileUnit
  integer :: iostat

  integer :: k
  integer :: m


  namelist /file_nml/ &
    & InASCIIFn, OutNcFn

  InASCIIFn   = ''
  OutNcFn     = ''

  call fi_open( CtlFN, "read", FileUnit )
  !
  rewind( FileUnit )
  read( FileUnit, nml = file_nml, iostat = iostat )
  if ( iostat > 0 ) then
    stop 'Namelist, file_nml, is something wrong.'
  end if
  write( 6, nml = file_nml )
  !
  close( FileUnit )


  !
  ! Input
  !

  ! ==========  P(NLev)  , T(NLev)  , VMR(NLev)    (NLev)-th level (upper boundary)
  ! -  -  -  -  Rad(NLev-1), MMR(NLev-1)           (NLev-1)-th layer
  ! ==========  P(NLev-1), T(NLev-1), VMR(NLev-1)  (NLev-1)-th level
  ! -  -  -  -  Rad(NLev-2), MMR(NLev-2)           (NLev-2)-th layer
  !
  ! ...
  !
  ! -  -  -  -  Rad(3), MMR(3)      2nd layer
  ! ==========  P(3), T(3), VMR(3)  3rd level
  ! -  -  -  -  Rad(2), MMR(2)      2nd layer
  ! ==========  P(2), T(2), VMR(2)  2nd level
  ! -  -  -  -  Rad(1), MMR(1)      1st layer
  ! ==========  P(1), T(1), VMR(1)  1st level
  ! ==========  P(1), Ts            ground    (lower boundary)
  !
  !
  ! NLev        : Number of levels (layer interfaces)
  ! NMol        : Number of molecules
  ! MolNumX     : Xth molecular number
  ! Ts          : surface temperature
  ! P(k)        : pressure at k'th level
  ! T(k)        : temperature at k'th level
  ! MolX_VMR(k) : volume mixing ratio of Xth molecule at k'th level
  ! NPtcl       : Number of particles
  ! PtclYName   : name of Yth particle
  ! PtclYDens   : density of Yth particle
  ! PtclYRad(l) : effective radius of of Yth particle at l'th layer
  ! PtclYMMR(l) : mass mixing ratio of of Yth particle at l'th layer


  ! l.1                        : Comment (This line is not read, actually.)
  ! l.2                        : NLev
  ! l.3                        : NMol
  ! l.4                        : MolNum1, MolNum2, ...
  !
  ! l.5                        : Ps
  ! l.6                        : P(1)   , T(1)   , Mol1_VMR(1)   , Mol2_VMR(1)   , ...
  ! l.7                        : P(2)   , T(2)   , Mol1_VMR(2)   , Mol2_VMR(2)   , ...
  ! ...
  ! l.(5+NLev)                 : P(NLev), T(NLev), Mol1_VMR(NLev), Mol2_VMR(NLev), ...
  !
  ! l.(5+NLev+1)               : NPtcl
  ! l.(5+NLev+2)               : Ptcl1Name, Ptcl2Name, ...
  ! l.(5+NLev+3)               : Ptcl1Dens, Ptcl2Dens, ...
  !
  ! l.(5+NLev+3+1)             : Ptcl1Rad(1)     , Ptcl2Rad(1)     , ...
  ! l.(5+NLev+3+2)             : Ptcl1Rad(2)     , Ptcl2Rad(2)     , ...
  ! ...
  ! l.(5+NLev+3+NLev-1)        : Ptcl1Rad(NLev-1), Ptcl2Rad(NLev-1), ...
  !
  ! l.(5+NLev+3+NLev-1+1)      : Ptcl1MMR(1)     , Ptcl2MMR(1) , ...
  ! l.(5+NLev+3+NLev-1+2)      : Ptcl1MMR(2)     , Ptcl2MMR(2) , ...
  ! ...
  ! l.(5+NLev+3+NLev-1+NLev-1) : Ptcl1MMR(NLev-1), Ptcl2MMR(NLev-1) , ...
  !
  call fi_open( InASCIIFN, "read", FileUnit )
  read( FileUnit, * )
  read( FileUnit, * ) NLev
  read( FileUnit, * ) NMol
  allocate( m_MolNum(NMol) )
  allocate( r_Press (NLev) )
  allocate( r_Temp  (NLev) )
  allocate( rm_VMR  (NLev, NMol) )
  read( FileUnit, * ) ( m_MolNum(m), m=1,NMol )
  read( FileUnit, * ) Ts
  do k = 1, NLev
    read( FileUnit, * ) r_Press(k), r_Temp(k), ( rm_VMR(k,m), m=1,NMol )
  end do
  read( FileUnit, * ) NPtcl
  allocate( a_PtclNum    (NPtcl) )
  allocate( a_PtclName   (NPtcl) )
  allocate( a_PtclDens   (NPtcl) )
  allocate( za_PtclEffRad(NLev-1, NPtcl) )
  allocate( za_PtclMMR   (NLev-1, NPtcl) )
  if ( NPtcl > 0 ) then
    do m = 1, NPtcl
      a_PtclNum(m) = m
    end do
    read( FileUnit, * ) ( a_PtclName  (m), m=1,NPtcl )
    read( FileUnit, * ) ( a_PtclDens  (m), m=1,NPtcl )
    do k = 1, NLev-1
      read( FileUnit, * ) ( za_PtclEffRad(k,m), m=1,NPtcl )
    end do
    do k = 1, NLev-1
      read( FileUnit, * ) ( za_PtclMMR(k,m), m=1,NPtcl )
    end do
  end if
  close( FileUnit )



  ! Set particles
  !
  allocate( z_Press(1:NLev-1) )
  do k = 1, NLev-1
    z_Press(k) = ( r_Press(k) + r_Press(k+1) ) / 2.0d0
  end do


  Ps = r_Press(1)

  call IOProfileOutput( &
    & OutNcFn, &
    & NLev, NMol, r_Press, m_MolNum, &
    & r_Temp, rm_VMR, Ps, Ts, &
    & z_Press, &
    & NChar, NPtcl, a_PtclNum, a_PtclName, &
    & a_PtclDens, za_PtclEffRad, za_PtclMMR &
    & )


  write( 6, * ) 'Finish'

end program main

!------------------------------------------------------------------------------
