!------------------------------------------------------------------------------

subroutine CalcHeight( &
  & kmax,                                   &
  & r_Press, r_Temp, r_VMRH2O, r_MeanMolWt, &
  & r_Height                                &
  & )

  use vtype_module

  use constants0, only : &
    & GasRUniv

  use rad_planetconst, only : &
    & Grav,            &
    & MolWtH2O

  implicit none

  integer , intent(in ) :: kmax
  real(dp), intent(in ) :: r_Press    (0:kmax)
  real(dp), intent(in ) :: r_Temp     (0:kmax)
  real(dp), intent(in ) :: r_VMRH2O   (0:kmax)
  real(dp), intent(in ) :: r_MeanMolWt(0:kmax)
  real(dp), intent(out) :: r_Height   (0:kmax)


  ! Local variables
  !
  real(dp) :: r_GasR   (0:kmax)
  real(dp) :: r_EpsV   (0:kmax)
  real(dp) :: r_MMRH2O (0:kmax)
  real(dp) :: r_VirTemp(0:kmax)
  real(dp) :: PressTop
  integer  :: k


  r_GasR = GasRUniv / r_MeanMolWt
  r_EpsV = MolWtH2O / r_MeanMolWt

  r_MMRH2O = r_VMRH2O * MolWtH2O / r_MeanMolWt

  r_VirTemp = r_Temp &
    & * ( 1.0d0 + ( ( ( 1.0d0 / r_EpsV ) - 1.0d0 ) * r_MMRH2O ) )

  ! dp/dz = -rho * g = - p * g / ( R * T )
  ! 1/p * dp/dz = - g / ( R * T )
  ! d(lnp)/dz = - g / ( R * T )
  ! dz = - R * T / g * d(lnp)

  r_Height(0) = 0.0d0
  do k = 0+1, kmax
    if ( r_Press(k) <= 0.0d0 ) then
      PressTop = r_Press(k-1) * 1.0d-1
    else
      PressTop = r_Press(k)
    end if
    r_Height(k) = r_Height(k-1)                              &
      & -   ( r_GasR(k-1) + r_GasR(k) ) / 2.0d0              &
      &   * ( r_VirTemp(k-1) + r_VirTemp(k) ) / 2.0d0        &
      &   / Grav                                             &
      &   * log( PressTop / r_Press(k-1) )
  end do


end subroutine CalcHeight

!------------------------------------------------------------------------------

subroutine Filter( &
  & FilterCoef, &
  & kmax, r_Press, &
  & r_Temp, &
  & r_DelTemp &
  & )

  use vtype_module

  use rad_planetconst, only : &
    & Grav,            &
    & Cp

  implicit none

  real(DP), intent(in   ) :: FilterCoef
  integer , intent(in   ) :: kmax
  real(DP), intent(in   ) :: r_Press(0:kmax)
  real(DP), intent(inout) :: r_Temp     (0:kmax)
  real(DP), intent(  out) :: r_DelTemp  (0:kmax)


  ! Local variables
  !
  real(DP) :: r_TempB     (0:kmax)
  real(DP) :: z_DelAtmMass(1:kmax)
  real(DP) :: EneSumB
  real(DP) :: EneSum

  integer  :: k


  r_TempB = r_Temp

  do k = 1, kmax
    z_DelAtmMass(k) = ( r_Press(k-1) - r_Press(k) ) / Grav
  end do

  do k = 0+1, kmax-1
    if ( k == 1 ) then
      EneSumB =   0.0d0                              &
        &       + Cp * z_DelAtmMass(k) * r_Temp(k  ) &
        &       + Cp * z_DelAtmMass(k) * r_Temp(k+1)
    else
      EneSumB =   Cp * z_DelAtmMass(k) * r_Temp(k-1) &
        &       + Cp * z_DelAtmMass(k) * r_Temp(k  ) &
        &       + Cp * z_DelAtmMass(k) * r_Temp(k+1)
    end if
    r_Temp(k) = r_Temp(k) &
      & + FilterCoef * ( r_Temp(k-1) - 2.0d0 * r_Temp(k) + r_Temp(k+1) )
    if ( k == 1 ) then
      EneSum  =   0.0d0                              &
        &       + Cp * z_DelAtmMass(k) * r_Temp(k  ) &
        &       + Cp * z_DelAtmMass(k) * r_Temp(k+1)
      !
      r_Temp(k  ) = r_Temp(k  ) * EneSumB / EneSum
      r_Temp(k+1) = r_Temp(k+1) * EneSumB / EneSum
    else
      EneSum  =   Cp * z_DelAtmMass(k) * r_Temp(k-1) &
        &       + Cp * z_DelAtmMass(k) * r_Temp(k  ) &
        &       + Cp * z_DelAtmMass(k) * r_Temp(k+1)
      r_Temp(k-1) = r_Temp(k-1) * EneSumB / EneSum
      r_Temp(k  ) = r_Temp(k  ) * EneSumB / EneSum
      r_Temp(k+1) = r_Temp(k+1) * EneSumB / EneSum
    end if
  end do

  r_DelTemp = r_Temp - r_TempB


end subroutine Filter

!------------------------------------------------------------------------------
