module h2o_ice

  use vtype_module
  use fi_module

  implicit none

  private

  real(dp), parameter :: RhoIce = 0.9167d3

  integer              , save :: NSWData
  real(dp), allocatable, save :: a_SWWL1  (:)
  real(dp), allocatable, save :: a_SWWL2  (:)
  real(dp), allocatable, save :: a_SWExtCoef0(:)
  real(dp), allocatable, save :: a_SWExtCoef1(:)
  real(dp), allocatable, save :: a_SWSSCoA0(:)
  real(dp), allocatable, save :: a_SWSSCoA1(:)
  real(dp), allocatable, save :: a_SWSSCoA2(:)
  real(dp), allocatable, save :: a_SWSSCoA3(:)
  real(dp), allocatable, save :: a_SWAF0(:)
  real(dp), allocatable, save :: a_SWAF1(:)
  real(dp), allocatable, save :: a_SWAF2(:)
  real(dp), allocatable, save :: a_SWAF3(:)
  real(dp), allocatable, save :: a_SWDelFunc0(:)
  real(dp), allocatable, save :: a_SWDelFunc1(:)
  real(dp), allocatable, save :: a_SWDelFunc2(:)
  real(dp), allocatable, save :: a_SWDelFunc3(:)

  integer              , save :: NLWData
  real(dp), allocatable, save :: a_LWWL      (:)
  real(dp), allocatable, save :: a_LWExtCoef0(:)
  real(dp), allocatable, save :: a_LWExtCoef1(:)
  real(dp), allocatable, save :: a_LWExtCoef2(:)
  real(dp), allocatable, save :: a_LWAbsCoef0(:)
  real(dp), allocatable, save :: a_LWAbsCoef1(:)
  real(dp), allocatable, save :: a_LWAbsCoef2(:)
  real(dp), allocatable, save :: a_LWAbsCoef3(:)
  real(dp), allocatable, save :: a_LWAF0(:)
  real(dp), allocatable, save :: a_LWAF1(:)
  real(dp), allocatable, save :: a_LWAF2(:)
  real(dp), allocatable, save :: a_LWAF3(:)


  public :: H2OIceInit
  public :: H2OIceFinalize
  public :: H2OIceCalcParam


  !----------------------------------------------------------------------------
contains
  !----------------------------------------------------------------------------

  subroutine H2OIceInit( Dir )

    character(*), intent(in) :: Dir


    character(extstr) :: FileName
    character(stdstr) :: Mode = "read"

    integer           :: FU

    integer :: l

    integer           :: NHeader = 4



    FileName = trim( Dir ) // "/data_Fu1996_Table3a_Ice_ExtCoef_SW.txt"
    NSWData = fi_lc( FileName, NHeader )
    allocate( a_SWWL1     (NSWData) )
    allocate( a_SWWL2     (NSWData) )
    allocate( a_SWExtCoef0(NSWData) )
    allocate( a_SWExtCoef1(NSWData) )
    allocate( a_SWSSCoA0  (NSWData) )
    allocate( a_SWSSCoA1  (NSWData) )
    allocate( a_SWSSCoA2  (NSWData) )
    allocate( a_SWSSCoA3  (NSWData) )
    allocate( a_SWAF0     (NSWData) )
    allocate( a_SWAF1     (NSWData) )
    allocate( a_SWAF2     (NSWData) )
    allocate( a_SWAF3     (NSWData) )
    allocate( a_SWDelFunc0(NSWData) )
    allocate( a_SWDelFunc1(NSWData) )
    allocate( a_SWDelFunc2(NSWData) )
    allocate( a_SWDelFunc3(NSWData) )

    FileName = trim( Dir ) // "/data_Fu1996_Table3a_Ice_ExtCoef_SW.txt"
    call fi_open( FileName, Mode, FU )
    do l = 1, NHeader
      read( FU, * )
    end do
    do l = 1, NSWData
      read( FU, * ) a_SWWL1(l), a_SWWL2(l), &
        & a_SWExtCoef0(l), a_SWExtCoef1(l)
    end do
    close( FU )

    FileName = trim( Dir ) // "/data_Fu1996_Table3b_Ice_SSA_SW.txt"
    call fi_open( FileName, Mode, FU )
    do l = 1, NHeader
      read( FU, * )
    end do
    do l = 1, NSWData
      read( FU, * ) a_SWWL1(l), a_SWWL2(l), &
        & a_SWSSCoA0(l), a_SWSSCoA1(l), a_SWSSCoA2(l), a_SWSSCoA3(l)
    end do
    close( FU )

    FileName = trim( Dir ) // "/data_Fu1996_Table3c_Ice_AF_SW.txt"
    call fi_open( FileName, Mode, FU )
    do l = 1, NHeader
      read( FU, * )
    end do
    do l = 1, NSWData
      read( FU, * ) a_SWWL1(l), a_SWWL2(l), &
        & a_SWAF0(l), a_SWAF1(l), a_SWAF2(l), a_SWAF3(l)
    end do
    close( FU )

    FileName = trim( Dir ) // "/data_Fu1996_Table3d_Ice_Delta_SW.txt"
    call fi_open( FileName, Mode, FU )
    do l = 1, NHeader
      read( FU, * )
    end do
    do l = 1, NSWData
      read( FU, * ) a_SWWL1(l), a_SWWL2(l), &
        & a_SWDelFunc0(l), a_SWDelFunc1(l), a_SWDelFunc2(l), a_SWDelFunc3(l)
    end do
    close( FU )


    FileName = trim( Dir ) // "/data_Fu1998_Table1a_Ice_EC_LW.txt"
    NLWData = fi_lc( FileName, NHeader )
    allocate( a_LWWL      (NLWData) )
    allocate( a_LWExtCoef0(NLWData) )
    allocate( a_LWExtCoef1(NLWData) )
    allocate( a_LWExtCoef2(NLWData) )
    allocate( a_LWAbsCoef0(NLWData) )
    allocate( a_LWAbsCoef1(NLWData) )
    allocate( a_LWAbsCoef2(NLWData) )
    allocate( a_LWAbsCoef3(NLWData) )
    allocate( a_LWAF0     (NLWData) )
    allocate( a_LWAF1     (NLWData) )
    allocate( a_LWAF2     (NLWData) )
    allocate( a_LWAF3     (NLWData) )

    FileName = trim( Dir ) // "/data_Fu1998_Table1a_Ice_EC_LW.txt"
    call fi_open( FileName, Mode, FU )
    do l = 1, NHeader
      read( FU, * )
    end do
    do l = 1, NLWData
      read( FU, * ) a_LWWL(l), &
        & a_LWExtCoef0(l), a_LWExtCoef1(l), a_LWExtCoef2(l)
    end do
    close( FU )

    FileName = trim( Dir ) // "/data_Fu1998_Table1b_Ice_AC_LW.txt"
    call fi_open( FileName, Mode, FU )
    do l = 1, NHeader
      read( FU, * )
    end do
    do l = 1, NLWData
      read( FU, * ) a_LWWL(l), &
        & a_LWAbsCoef0(l), a_LWAbsCoef1(l), a_LWAbsCoef2(l), a_LWAbsCoef3(l)
    end do
    close( FU )

    FileName = trim( Dir ) // "/data_Fu1998_Table1c_Ice_AF_LW.txt"
    call fi_open( FileName, Mode, FU )
    do l = 1, NHeader
      read( FU, * )
    end do
    do l = 1, NLWData
      read( FU, * ) a_LWWL(l), &
        & a_LWAF0(l), a_LWAF1(l), a_LWAF2(l), a_LWAF3(l)
    end do
    close( FU )

  end subroutine H2OIceInit

  !----------------------------------------------------------------------------

  subroutine H2OIceFinalize


    deallocate( a_SWWL1      )
    deallocate( a_SWWL2      )
    deallocate( a_SWExtCoef0 )
    deallocate( a_SWExtCoef1 )
    deallocate( a_SWSSCoA0   )
    deallocate( a_SWSSCoA1   )
    deallocate( a_SWSSCoA2   )
    deallocate( a_SWSSCoA3   )
    deallocate( a_SWAF0      )
    deallocate( a_SWAF1      )
    deallocate( a_SWAF2      )
    deallocate( a_SWAF3      )
    deallocate( a_SWDelFunc0 )
    deallocate( a_SWDelFunc1 )
    deallocate( a_SWDelFunc2 )
    deallocate( a_SWDelFunc3 )


    deallocate( a_LWWL       )
    deallocate( a_LWExtCoef0 )
    deallocate( a_LWExtCoef1 )
    deallocate( a_LWExtCoef2 )
    deallocate( a_LWAbsCoef0 )
    deallocate( a_LWAbsCoef1 )
    deallocate( a_LWAbsCoef2 )
    deallocate( a_LWAbsCoef3 )
    deallocate( a_LWAF0      )
    deallocate( a_LWAF1      )
    deallocate( a_LWAF2      )
    deallocate( a_LWAF3      )


  end subroutine H2OIceFinalize

  !----------------------------------------------------------------------------

  subroutine H2OIceCalcParam( &
    & PartSize, Wavelength, &
    & QExt, SSA, AF &
    & )

    real(dp), intent(in ) :: PartSize
    real(dp), intent(in ) :: Wavelength
    real(dp), intent(out) :: QExt
    real(dp), intent(out) :: SSA
    real(dp), intent(out) :: AF


    ! Local variables
    !
    real(dp) :: WavelengthMicron
    real(dp) :: PartSizeMicron
    real(dp) :: ExtCoef0
    real(dp) :: ExtCoef1
    real(dp) :: ExtCoef2
    real(dp) :: AbsCoef0
    real(dp) :: AbsCoef1
    real(dp) :: AbsCoef2
    real(dp) :: AbsCoef3
    real(dp) :: AF0
    real(dp) :: AF1
    real(dp) :: AF2
    real(dp) :: AF3

    real(dp) :: SSCoA
    real(dp) :: QAbs
    real(dp) :: DelFunc

    integer  :: l


    PartSizeMicron   = PartSize   * 1.0d6
    WavelengthMicron = Wavelength * 1.0d6

    if ( WavelengthMicron <= a_SWWL2(NSWData) ) then
      ! short wave

      ! check particle size
      if ( ( PartSizeMicron < 8.75d0 ) .or. ( 3100.d0 < PartSizeMicron ) ) then
        write( 6, * ) 'ERROR : Particle size is out of range:'
        write( 6, * ) '   Wavelength          : ', WavelengthMicron, ' micron'
        write( 6, * ) '   Particle size       : ', PartSizeMicron, ' micron'
        write( 6, * ) '   Particle size range : 8.75 <= L <= 3100 micron'
        stop
      end if

      do l = 1, NSWData
        if ( WavelengthMicron <= a_SWWL2(l) ) exit
      end do

      QExt  =   a_SWExtCoef0(l) &
        &     + a_SWExtCoef1(l) / PartSizeMicron
      QExt  = QExt * 4.0d0 / 3.0d0 * RhoIce * PartSize * 1.0d3
      SSCoA =   a_SWSSCoA0(l) &
        &     + a_SWSSCoA1(l) * PartSizeMicron &
        &     + a_SWSSCoA2(l) * PartSizeMicron**2 &
        &     + a_SWSSCoA3(l) * PartSizeMicron**3
      SSA   = 1.0d0 - SSCoA
      AF    =   a_SWAF0(l) &
        &     + a_SWAF1(l) * PartSizeMicron &
        &     + a_SWAF2(l) * PartSizeMicron**2 &
        &     + a_SWAF3(l) * PartSizeMicron**3
      DelFunc =   a_SWDelFunc0(l) &
        &       + a_SWDelFunc1(l) * PartSizeMicron &
        &       + a_SWDelFunc2(l) * PartSizeMicron**2 &
        &       + a_SWDelFunc3(l) * PartSizeMicron**3


      QExt  = max( QExt, 0.0d0 )
      SSA   = max( SSA , 0.0d0 )
      AF    = max( AF  , 0.0d0 )
!!$      DelFunc = max( DelFunc, 0.0d0 )

    else
      ! long wave

      ! check particle size
!!$      if ( ( PartSizeMicron < 2.0d0 ) .or. ( 37.5d0 < PartSizeMicron ) ) then
!!$        write( 6, * ) 'ERROR : Particle size is out of range:'
!!$        write( 6, * ) '   Wavelength          : ', WavelengthMicron, ' micron'
!!$        write( 6, * ) '   Particle size       : ', PartSizeMicron, ' micron'
!!$        write( 6, * ) '   Particle size range : 2.0 <= L <= 37.5 micron'
!!$        stop
!!$      end if
      if ( ( PartSizeMicron < 11.0d0 ) .or. ( 129.6d0 < PartSizeMicron ) ) then
        write( 6, * ) 'ERROR : Particle size is out of range:'
        write( 6, * ) '   Wavelength          : ', WavelengthMicron, ' micron'
        write( 6, * ) '   Particle size       : ', PartSizeMicron, ' micron'
        write( 6, * ) '   Particle size range : 11.0 <= L <= 129.6 micron'
        stop
      end if

      if ( WavelengthMicron <= a_LWWL(1) ) then
        l = 1
        ExtCoef0 = a_LWExtCoef0(l)
        ExtCoef1 = a_LWExtCoef1(l)
        ExtCoef2 = a_LWExtCoef2(l)
        AbsCoef0 = a_LWAbsCoef0(l)
        AbsCoef1 = a_LWAbsCoef1(l)
        AbsCoef2 = a_LWAbsCoef2(l)
        AbsCoef3 = a_LWAbsCoef3(l)
        AF0      = a_LWAF0(l)
        AF1      = a_LWAF1(l)
        AF2      = a_LWAF2(l)
        AF3      = a_LWAF3(l)
      else if ( a_LWWL(NLWData) <= WavelengthMicron ) then
        l = NLWData
        ExtCoef0 = a_LWExtCoef0(l)
        ExtCoef1 = a_LWExtCoef1(l)
        ExtCoef2 = a_LWExtCoef2(l)
        AbsCoef0 = a_LWAbsCoef0(l)
        AbsCoef1 = a_LWAbsCoef1(l)
        AbsCoef2 = a_LWAbsCoef2(l)
        AbsCoef3 = a_LWAbsCoef3(l)
        AF0      = a_LWAF0(l)
        AF1      = a_LWAF1(l)
        AF2      = a_LWAF2(l)
        AF3      = a_LWAF3(l)
      else

        do l = 2, NLWData
          if ( WavelengthMicron <= a_LWWL(l) ) exit
        end do
        ExtCoef0 = LinearInterpolate( a_LWWL(l-1:l), a_LWExtCoef0(l-1:l), WavelengthMicron )
        ExtCoef1 = LinearInterpolate( a_LWWL(l-1:l), a_LWExtCoef1(l-1:l), WavelengthMicron )
        ExtCoef2 = LinearInterpolate( a_LWWL(l-1:l), a_LWExtCoef2(l-1:l), WavelengthMicron )
        AbsCoef0 = LinearInterpolate( a_LWWL(l-1:l), a_LWAbsCoef0(l-1:l), WavelengthMicron )
        AbsCoef1 = LinearInterpolate( a_LWWL(l-1:l), a_LWAbsCoef1(l-1:l), WavelengthMicron )
        AbsCoef2 = LinearInterpolate( a_LWWL(l-1:l), a_LWAbsCoef2(l-1:l), WavelengthMicron )
        AbsCoef3 = LinearInterpolate( a_LWWL(l-1:l), a_LWAbsCoef3(l-1:l), WavelengthMicron )
        AF0      = LinearInterpolate( a_LWWL(l-1:l), a_LWAF0(l-1:l), WavelengthMicron )
        AF1      = LinearInterpolate( a_LWWL(l-1:l), a_LWAF1(l-1:l), WavelengthMicron )
        AF2      = LinearInterpolate( a_LWWL(l-1:l), a_LWAF2(l-1:l), WavelengthMicron )
        AF3      = LinearInterpolate( a_LWWL(l-1:l), a_LWAF3(l-1:l), WavelengthMicron )
      end if

      QExt =   ExtCoef0 &
        &    + ExtCoef1 / PartSizeMicron &
        &    + ExtCoef2 / PartSizeMicron**2
      QAbs =   AbsCoef0 &
        &    + AbsCoef1 * PartSizeMicron &
        &    + AbsCoef2 * PartSizeMicron**2 &
        &    + AbsCoef3 * PartSizeMicron**3
      QAbs = QAbs / PartSizeMicron
      SSA  = 1.0d0 - QAbs / QExt
      AF   =   AF0 &
        &    + AF1 * PartSizeMicron &
        &    + AF2 * PartSizeMicron**2 &
        &    + AF3 * PartSizeMicron**3
      QExt  = QExt * 4.0d0 / 3.0d0 * RhoIce * PartSize * 1.0d3

    end if


    QExt  = max( QExt, 0.0d0 )
    SSA   = max( SSA , 0.0d0 )
    AF    = max( AF  , 0.0d0 )
!!$    DelFunc = max( DelFunc, 0.0d0 )


  end subroutine H2OIceCalcParam

  !----------------------------------------------------------------------------

  function LinearInterpolate( a_X, a_Y, X ) result( Y )

    real(dp), intent(in ) :: a_X(2)
    real(dp), intent(in ) :: a_Y(2)
    real(dp), intent(in ) :: X

    real(dp) :: Y


    Y = ( a_Y(2) - a_Y(1) ) / ( a_X(2) - a_X(1) ) * ( X - a_X(1) ) + a_Y(1)

  end function LinearInterpolate

  !----------------------------------------------------------------------------

end module h2o_ice
