module saturation_H2O_Dennou

  use vtype_module

  implicit none

  real(DP), parameter :: LatentHeat       = 2.5d6
  real(DP), parameter :: LatentHeatFusion = 334.0d3

  public :: PressH2OVapSat
  public :: DPressH2OVapSatDTemp

contains

  !----------------------------------------------------------------------------

  function PressH2OVapSat( Temp ) result( PressH2OVapSat )

    use constants0, only : GasRUniv

    real(DP), intent(in ) :: Temp

    real(DP) :: PressH2OVapSat


    ! NHA1992
!!$    PressH2OVapSat = 1.4d11 * exp( - LatentHeatPerMol / ( GasRUniv * Temp ) )

    ! Dennou
    if ( Temp < 273.0d0 ) then
      PressH2OVapSat = &
        & 611.0 * exp( ( LatentHeat + LatentHeatFusion ) / ( GasRUniv / 18.0d-3 ) * ( 1.0/273.0 - 1.0/Temp ) )
    else
      PressH2OVapSat = &
        & 611.0 * exp( LatentHeat / ( GasRUniv / 18.0d-3 ) * ( 1.0/273.0 - 1.0/Temp ) )
    end if

  end function PressH2OVapSat

  !----------------------------------------------------------------------------

  function DPressH2OVapSatDTemp( Temp ) result( DPressH2OVapSatDTemp )

    use constants0, only : GasRUniv

    real(DP), intent(in ) :: Temp

    real(DP) :: DPressH2OVapSatDTemp


    ! NHA1992
!!$    DPressH2OVapSatDTemp = &
!!$      & 1.4d11 * exp( - LatentHeatPerMol / ( GasRUniv * Temp ) ) &
!!$      & * LatentHeatPerMol / ( GasRUniv * Temp**2 )

    ! Dennou
    if ( Temp < 273.0d0 ) then
      DPressH2OVapSatDTemp = &
        & 611.0 * exp( ( LatentHeat + LatentHeatFusion ) / ( GasRUniv / 18.0d-3 ) * ( 1.0/273.0 - 1.0/Temp ) ) &
        & * ( LatentHeat + LatentHeatFusion ) / ( GasRUniv / 18.0d-3 * Temp**2 )
    else
      DPressH2OVapSatDTemp = &
        & 611.0 * exp( LatentHeat / ( GasRUniv / 18.0d-3 ) * ( 1.0/273.0 - 1.0/Temp ) ) &
        & * LatentHeat / ( GasRUniv / 18.0d-3 * Temp**2 )
    end if


  end function DPressH2OVapSatDTemp

  !----------------------------------------------------------------------------

end module saturation_H2O_Dennou
