module saturation_K1991

  use vtype_module

  implicit none

  public :: SatK1991CalcPressCO2VapSat
  public :: SatK1991CalcDPressCO2VapSatDTemp
  public :: SatK1991CalcTempSat

contains

  !----------------------------------------------------------------------------

  function SatK1991CalcPressCO2VapSat( Temp ) result( PressCO2VapSat )

    real(DP), intent(in ) :: Temp

    real(DP) :: PressCO2VapSat


    if ( Temp > 216.56d0 ) then
      PressCO2VapSat = &
        &   3.128082d0 &
        & - 867.2124 / Temp &
        & + 18.65612d-3 * Temp &
        & - 72.48820d-6 * Temp * Temp &
        & + 93.0d-9 * Temp * Temp * Temp
    else
      PressCO2VapSat = &
        &   6.760956d0 &
        & - 1284.07 / ( Temp - 4.718d0 ) &
        & + 1.256d-4 * ( Temp - 143.15d0 )
    end if
    PressCO2VapSat = 10.0d0**PressCO2VapSat
    PressCO2VapSat = PressCO2VapSat * 101325.0d0


  end function SatK1991CalcPressCO2VapSat

  !----------------------------------------------------------------------------

  function SatK1991CalcDPressCO2VapSatDTemp( Temp ) result( DPressCO2VapSatDTemp )

    real(DP), intent(in ) :: Temp

    real(DP) :: DPressCO2VapSatDTemp


    if ( Temp > 216.56d0 ) then
      DPressCO2VapSatDTemp = &
        & + 867.2124 / ( Temp * Temp ) &
        & + 18.65612d-3 &
        & - 72.48820d-6 * 2.0d0 * Temp &
        & + 93.0d-9 * 3.0d0 * Temp * Temp
    else
      DPressCO2VapSatDTemp = &
        & + 1284.07 / ( ( Temp - 4.718d0 ) * ( Temp - 4.718d0 ) ) &
        & + 1.256d-4
    end if
    DPressCO2VapSatDTemp = DPressCO2VapSatDTemp &
      * SatK1991CalcPressCO2VapSat( Temp )


  end function SatK1991CalcDPressCO2VapSatDTemp

  !----------------------------------------------------------------------------

  function SatK1991CalcTempSat( Press ) result( SatTemp )

    real(DP), intent(in ) :: Press

    real(DP) :: SatTemp

    ! Local variables
    !
    real(DP) :: Temp1
    real(DP) :: Temp2
    real(DP) :: TempMP
    real(DP) :: DelTemp
    real(DP) :: PressCO2VapSat1
    real(DP) :: PressCO2VapSat2
    real(DP) :: PressCO2VapSatMP


    ! Bisection method
    Temp1 = 200.0d0
    PressCO2VapSat1 = SatK1991CalcPressCO2VapSat( Temp1 )
    if ( PressCO2VapSat1 >= Press ) then
      DelTemp = -20.0d0
      do
        Temp1 = Temp1 + DelTemp
        PressCO2VapSat1 = SatK1991CalcPressCO2VapSat( Temp1 )
        if ( PressCO2VapSat1 < Press ) exit
      end do
    end if
    DelTemp = 20.0d0
    do
      Temp2 = Temp1 + DelTemp
      PressCO2VapSat2 = SatK1991CalcPressCO2VapSat( Temp2 )
      if ( PressCO2VapSat2 >= Press ) exit
      Temp1 = Temp2
    end do
    do
      TempMP = ( Temp1 + Temp2 ) / 2.0d0
      PressCO2VapSatMP = SatK1991CalcPressCO2VapSat( TempMP )
      if ( PressCO2VapSatMP >= Press ) then
        Temp2 = TempMP
      else
        Temp1 = TempMP
      end if
      if ( abs( Temp2 - Temp1 ) < 1.0d-5 ) then
        exit
      end if
    end do

    SatTemp = Temp1

  end function SatK1991CalcTempSat

  !----------------------------------------------------------------------------

end module saturation_K1991
